@@ -15,13 +15,17 @@
 # specific language governing permissions and limitations
 # under the License.
 """This module contains the Apache Livy operator."""
+
 from __future__ import annotations
 
-from time import sleep
+import time
+from functools import cached_property
 from typing import TYPE_CHECKING, Any, Sequence
 
+from deprecated.classic import deprecated
+
 from airflow.configuration import conf
-from airflow.exceptions import AirflowException
+from airflow.exceptions import AirflowException, AirflowProviderDeprecationWarning
 from airflow.models import BaseOperator
 from airflow.providers.apache.livy.hooks.livy import BatchState, LivyHook
 from airflow.providers.apache.livy.triggers.livy import LivyTrigger
@@ -94,7 +98,8 @@ class LivyOperator(BaseOperator):
     ) -> None:
         super().__init__(**kwargs)
 
-        self.spark_params = {
+        spark_params = {
+            # Prepare spark parameters, it will be templated later.
             "file": file,
             "class_name": class_name,
             "args": args,
@@ -112,48 +117,50 @@ class LivyOperator(BaseOperator):
             "conf": conf,
             "proxy_user": proxy_user,
         }
-
+        self.spark_params = spark_params
         self._livy_conn_id = livy_conn_id
         self._livy_conn_auth_type = livy_conn_auth_type
         self._polling_interval = polling_interval
         self._extra_options = extra_options or {}
         self._extra_headers = extra_headers or {}
 
-        self._livy_hook: LivyHook | None = None
         self._batch_id: int | str
         self.retry_args = retry_args
         self.deferrable = deferrable
 
-    def get_hook(self) -> LivyHook:
+    @cached_property
+    def hook(self) -> LivyHook:
         """
         Get valid hook.
 
-        :return: hook
+        :return: LivyHook
         """
-        if self._livy_hook is None or not isinstance(self._livy_hook, LivyHook):
-            self._livy_hook = LivyHook(
-                livy_conn_id=self._livy_conn_id,
-                extra_headers=self._extra_headers,
-                extra_options=self._extra_options,
-                auth_type=self._livy_conn_auth_type,
-            )
-        return self._livy_hook
+        return LivyHook(
+            livy_conn_id=self._livy_conn_id,
+            extra_headers=self._extra_headers,
+            extra_options=self._extra_options,
+            auth_type=self._livy_conn_auth_type,
+        )
+
+    @deprecated(reason="use `hook` property instead.", category=AirflowProviderDeprecationWarning)
+    def get_hook(self) -> LivyHook:
+        """Get valid hook."""
+        return self.hook
 
     def execute(self, context: Context) -> Any:
-        self._batch_id = self.get_hook().post_batch(**self.spark_params)
+        self._batch_id = self.hook.post_batch(**self.spark_params)
         self.log.info("Generated batch-id is %s", self._batch_id)
 
         # Wait for the job to complete
         if not self.deferrable:
             if self._polling_interval > 0:
                 self.poll_for_termination(self._batch_id)
-            context["ti"].xcom_push(key="app_id", value=self.get_hook().get_batch(self._batch_id)["appId"])
+            context["ti"].xcom_push(key="app_id", value=self.hook.get_batch(self._batch_id)["appId"])
             return self._batch_id
 
-        hook = self.get_hook()
-        state = hook.get_batch_state(self._batch_id, retry_args=self.retry_args)
+        state = self.hook.get_batch_state(self._batch_id, retry_args=self.retry_args)
         self.log.debug("Batch with id %s is in state: %s", self._batch_id, state.value)
-        if state not in hook.TERMINAL_STATES:
+        if state not in self.hook.TERMINAL_STATES:
             self.defer(
                 timeout=self.execution_timeout,
                 trigger=LivyTrigger(
@@ -168,11 +175,11 @@ class LivyOperator(BaseOperator):
             )
         else:
             self.log.info("Batch with id %s terminated with state: %s", self._batch_id, state.value)
-            hook.dump_batch_logs(self._batch_id)
+            self.hook.dump_batch_logs(self._batch_id)
             if state != BatchState.SUCCESS:
                 raise AirflowException(f"Batch {self._batch_id} did not succeed")
 
-            context["ti"].xcom_push(key="app_id", value=self.get_hook().get_batch(self._batch_id)["appId"])
+            context["ti"].xcom_push(key="app_id", value=self.hook.get_batch(self._batch_id)["appId"])
             return self._batch_id
 
     def poll_for_termination(self, batch_id: int | str) -> None:
@@ -181,14 +188,13 @@ class LivyOperator(BaseOperator):
 
         :param batch_id: id of the batch session to monitor.
         """
-        hook = self.get_hook()
-        state = hook.get_batch_state(batch_id, retry_args=self.retry_args)
-        while state not in hook.TERMINAL_STATES:
+        state = self.hook.get_batch_state(batch_id, retry_args=self.retry_args)
+        while state not in self.hook.TERMINAL_STATES:
             self.log.debug("Batch with id %s is in state: %s", batch_id, state.value)
-            sleep(self._polling_interval)
-            state = hook.get_batch_state(batch_id, retry_args=self.retry_args)
+            time.sleep(self._polling_interval)
+            state = self.hook.get_batch_state(batch_id, retry_args=self.retry_args)
         self.log.info("Batch with id %s terminated with state: %s", batch_id, state.value)
-        hook.dump_batch_logs(batch_id)
+        self.hook.dump_batch_logs(batch_id)
         if state != BatchState.SUCCESS:
             raise AirflowException(f"Batch {batch_id} did not succeed")
 
@@ -198,11 +204,11 @@ class LivyOperator(BaseOperator):
     def kill(self) -> None:
         """Delete the current batch session."""
         if self._batch_id is not None:
-            self.get_hook().delete_batch(self._batch_id)
+            self.hook.delete_batch(self._batch_id)
 
     def execute_complete(self, context: Context, event: dict[str, Any]) -> Any:
         """
-        Callback for when the trigger fires - returns immediately.
+        Execute when the trigger fires - returns immediately.
 
         Relies on trigger to throw an exception, otherwise it assumes execution was successful.
         """
@@ -218,5 +224,5 @@ class LivyOperator(BaseOperator):
             self.task_id,
             event["response"],
         )
-        context["ti"].xcom_push(key="app_id", value=self.get_hook().get_batch(event["batch_id"])["appId"])
+        context["ti"].xcom_push(key="app_id", value=self.hook.get_batch(event["batch_id"])["appId"])
         return event["batch_id"]
