@@ -16,16 +16,18 @@
 # specific language governing permissions and limitations
 # under the License.
 """This module contains a Google Cloud Dataflow sensor."""
+
 from __future__ import annotations
 
 from typing import TYPE_CHECKING, Callable, Sequence
 
-from airflow.exceptions import AirflowException
+from airflow.exceptions import AirflowException, AirflowSkipException
 from airflow.providers.google.cloud.hooks.dataflow import (
     DEFAULT_DATAFLOW_LOCATION,
     DataflowHook,
     DataflowJobStatus,
 )
+from airflow.providers.google.common.hooks.base_google import PROVIDE_PROJECT_ID
 from airflow.sensors.base import BaseSensorOperator
 
 if TYPE_CHECKING:
@@ -66,7 +68,7 @@ class DataflowJobStatusSensor(BaseSensorOperator):
         *,
         job_id: str,
         expected_statuses: set[str] | str,
-        project_id: str | None = None,
+        project_id: str = PROVIDE_PROJECT_ID,
         location: str = DEFAULT_DATAFLOW_LOCATION,
         gcp_conn_id: str = "google_cloud_default",
         impersonation_chain: str | Sequence[str] | None = None,
@@ -106,7 +108,11 @@ class DataflowJobStatusSensor(BaseSensorOperator):
         if job_status in self.expected_statuses:
             return True
         elif job_status in DataflowJobStatus.TERMINAL_STATES:
-            raise AirflowException(f"Job with id '{self.job_id}' is already in terminal state: {job_status}")
+            # TODO: remove this if check when min_airflow_version is set to higher than 2.7.1
+            message = f"Job with id '{self.job_id}' is already in terminal state: {job_status}"
+            if self.soft_fail:
+                raise AirflowSkipException(message)
+            raise AirflowException(message)
 
         return False
 
@@ -148,7 +154,7 @@ class DataflowJobMetricsSensor(BaseSensorOperator):
         job_id: str,
         callback: Callable[[dict], bool],
         fail_on_terminal_state: bool = True,
-        project_id: str | None = None,
+        project_id: str = PROVIDE_PROJECT_ID,
         location: str = DEFAULT_DATAFLOW_LOCATION,
         gcp_conn_id: str = "google_cloud_default",
         impersonation_chain: str | Sequence[str] | None = None,
@@ -178,9 +184,11 @@ class DataflowJobMetricsSensor(BaseSensorOperator):
             )
             job_status = job["currentState"]
             if job_status in DataflowJobStatus.TERMINAL_STATES:
-                raise AirflowException(
-                    f"Job with id '{self.job_id}' is already in terminal state: {job_status}"
-                )
+                # TODO: remove this if check when min_airflow_version is set to higher than 2.7.1
+                message = f"Job with id '{self.job_id}' is already in terminal state: {job_status}"
+                if self.soft_fail:
+                    raise AirflowSkipException(message)
+                raise AirflowException(message)
 
         result = self.hook.fetch_job_metrics_by_id(
             job_id=self.job_id,
@@ -227,7 +235,7 @@ class DataflowJobMessagesSensor(BaseSensorOperator):
         job_id: str,
         callback: Callable,
         fail_on_terminal_state: bool = True,
-        project_id: str | None = None,
+        project_id: str = PROVIDE_PROJECT_ID,
         location: str = DEFAULT_DATAFLOW_LOCATION,
         gcp_conn_id: str = "google_cloud_default",
         impersonation_chain: str | Sequence[str] | None = None,
@@ -257,9 +265,11 @@ class DataflowJobMessagesSensor(BaseSensorOperator):
             )
             job_status = job["currentState"]
             if job_status in DataflowJobStatus.TERMINAL_STATES:
-                raise AirflowException(
-                    f"Job with id '{self.job_id}' is already in terminal state: {job_status}"
-                )
+                # TODO: remove this if check when min_airflow_version is set to higher than 2.7.1
+                message = f"Job with id '{self.job_id}' is already in terminal state: {job_status}"
+                if self.soft_fail:
+                    raise AirflowSkipException(message)
+                raise AirflowException(message)
 
         result = self.hook.fetch_job_messages_by_id(
             job_id=self.job_id,
@@ -306,7 +316,7 @@ class DataflowJobAutoScalingEventsSensor(BaseSensorOperator):
         job_id: str,
         callback: Callable,
         fail_on_terminal_state: bool = True,
-        project_id: str | None = None,
+        project_id: str = PROVIDE_PROJECT_ID,
         location: str = DEFAULT_DATAFLOW_LOCATION,
         gcp_conn_id: str = "google_cloud_default",
         impersonation_chain: str | Sequence[str] | None = None,
@@ -336,9 +346,11 @@ class DataflowJobAutoScalingEventsSensor(BaseSensorOperator):
             )
             job_status = job["currentState"]
             if job_status in DataflowJobStatus.TERMINAL_STATES:
-                raise AirflowException(
-                    f"Job with id '{self.job_id}' is already in terminal state: {job_status}"
-                )
+                # TODO: remove this if check when min_airflow_version is set to higher than 2.7.1
+                message = f"Job with id '{self.job_id}' is already in terminal state: {job_status}"
+                if self.soft_fail:
+                    raise AirflowSkipException(message)
+                raise AirflowException(message)
 
         result = self.hook.fetch_job_autoscaling_events_by_id(
             job_id=self.job_id,
