@@ -16,6 +16,7 @@
 # specific language governing permissions and limitations
 # under the License.
 """This module contains a Google Cloud Storage operator."""
+
 from __future__ import annotations
 
 import warnings
@@ -101,13 +102,13 @@ class GCSToGCSOperator(BaseOperator):
     ``copied_sales/2017/january-backup.avro`` in the ``data_backup`` bucket ::
 
         copy_single_file = GCSToGCSOperator(
-            task_id='copy_single_file',
-            source_bucket='data',
-            source_objects=['sales/sales-2017/january.avro'],
-            destination_bucket='data_backup',
-            destination_object='copied_sales/2017/january-backup.avro',
+            task_id="copy_single_file",
+            source_bucket="data",
+            source_objects=["sales/sales-2017/january.avro"],
+            destination_bucket="data_backup",
+            destination_object="copied_sales/2017/january-backup.avro",
             exact_match=True,
-            gcp_conn_id=google_cloud_conn_id
+            gcp_conn_id=google_cloud_conn_id,
         )
 
     The following Operator would copy all the Avro files from ``sales/sales-2017``
@@ -141,12 +142,12 @@ class GCSToGCSOperator(BaseOperator):
     process. ::
 
         move_files = GCSToGCSOperator(
-            task_id='move_files',
-            source_bucket='data',
-            source_object='sales/sales-2017/*.avro',
-            destination_bucket='data_backup',
+            task_id="move_files",
+            source_bucket="data",
+            source_object="sales/sales-2017/*.avro",
+            destination_bucket="data_backup",
             move_object=True,
-            gcp_conn_id=google_cloud_conn_id
+            gcp_conn_id=google_cloud_conn_id,
         )
 
     The following Operator would move all the Avro files from ``sales/sales-2019``
@@ -154,13 +155,13 @@ class GCSToGCSOperator(BaseOperator):
      ``data_backup`` bucket, deleting the original files in the process. ::
 
         move_files = GCSToGCSOperator(
-            task_id='move_files',
-            source_bucket='data',
-            source_objects=['sales/sales-2019/*.avro', 'sales/sales-2020'],
-            destination_bucket='data_backup',
-            delimiter='.avro',
+            task_id="move_files",
+            source_bucket="data",
+            source_objects=["sales/sales-2019/*.avro", "sales/sales-2020"],
+            destination_bucket="data_backup",
+            delimiter=".avro",
             move_object=True,
-            gcp_conn_id=google_cloud_conn_id
+            gcp_conn_id=google_cloud_conn_id,
         )
 
     """
@@ -207,7 +208,7 @@ class GCSToGCSOperator(BaseOperator):
                 stacklevel=2,
             )
         self.source_object = source_object
-        if source_objects and any([WILDCARD in obj for obj in source_objects]):
+        if source_objects and any(WILDCARD in obj for obj in source_objects):
             warnings.warn(
                 "Usage of wildcard (*) in 'source_objects' is deprecated, utilize 'match_glob' instead",
                 AirflowProviderDeprecationWarning,
@@ -237,7 +238,6 @@ class GCSToGCSOperator(BaseOperator):
         self.resolved_target_objects: set[str] = set()
 
     def execute(self, context: Context):
-
         hook = GCSHook(
             gcp_conn_id=self.gcp_conn_id,
             impersonation_chain=self.impersonation_chain,
@@ -277,7 +277,6 @@ class GCSToGCSOperator(BaseOperator):
         for prefix in self.source_objects:
             # Check if prefix contains wildcard
             if WILDCARD in prefix:
-
                 self._copy_source_with_wildcard(hook=hook, prefix=prefix)
             # Now search with prefix using provided delimiter if any
             else:
@@ -331,12 +330,12 @@ class GCSToGCSOperator(BaseOperator):
         the ``data_backup`` bucket (b/a.csv, b/b.csv, b/c.csv) ::
 
             copy_files = GCSToGCSOperator(
-                task_id='copy_files_without_wildcard',
-                source_bucket='data',
-                source_objects=['a/'],
-                destination_bucket='data_backup',
-                destination_object='b/',
-                gcp_conn_id=google_cloud_conn_id
+                task_id="copy_files_without_wildcard",
+                source_bucket="data",
+                source_objects=["a/"],
+                destination_bucket="data_backup",
+                destination_object="b/",
+                gcp_conn_id=google_cloud_conn_id,
             )
 
         Example 2:
@@ -347,13 +346,13 @@ class GCSToGCSOperator(BaseOperator):
         the ``data_backup`` bucket (b/a.avro, b/b.avro, b/c.avro) ::
 
             copy_files = GCSToGCSOperator(
-                task_id='copy_files_without_wildcard',
-                source_bucket='data',
-                source_objects=['a/'],
-                destination_bucket='data_backup',
-                destination_object='b/',
-                delimiter='.avro',
-                gcp_conn_id=google_cloud_conn_id
+                task_id="copy_files_without_wildcard",
+                source_bucket="data",
+                source_objects=["a/"],
+                destination_bucket="data_backup",
+                destination_object="b/",
+                delimiter=".avro",
+                gcp_conn_id=google_cloud_conn_id,
             )
 
         Example 3:
@@ -364,12 +363,12 @@ class GCSToGCSOperator(BaseOperator):
         the ``data_backup`` bucket (b/file_1.txt, b/file_2.csv, b/file_3.avro) ::
 
             copy_files = GCSToGCSOperator(
-                task_id='copy_files_without_wildcard',
-                source_bucket='data',
-                source_objects=['a/file_1.txt', 'a/file_2.csv', 'a/file_3.avro'],
-                destination_bucket='data_backup',
-                destination_object='b/',
-                gcp_conn_id=google_cloud_conn_id
+                task_id="copy_files_without_wildcard",
+                source_bucket="data",
+                source_objects=["a/file_1.txt", "a/file_2.csv", "a/file_3.avro"],
+                destination_bucket="data_backup",
+                destination_object="b/",
+                gcp_conn_id=google_cloud_conn_id,
             )
 
         Example 4:
@@ -380,12 +379,12 @@ class GCSToGCSOperator(BaseOperator):
         (b/foo.txt, b/foo.txt.abc, b/foo.txt/subfolder/file.txt) ::
 
             copy_files = GCSToGCSOperator(
-                task_id='copy_files_without_wildcard',
-                source_bucket='data',
-                source_object='a/foo.txt',
-                destination_bucket='data_backup',
-                destination_object='b/',
-                gcp_conn_id=google_cloud_conn_id
+                task_id="copy_files_without_wildcard",
+                source_bucket="data",
+                source_object="a/foo.txt",
+                destination_bucket="data_backup",
+                destination_object="b/",
+                gcp_conn_id=google_cloud_conn_id,
             )
         """
         objects = hook.list(
@@ -429,7 +428,7 @@ class GCSToGCSOperator(BaseOperator):
         # Check whether the prefix is a root directory for all the rest of objects.
         _pref = prefix.rstrip("/")
         is_directory = prefix.endswith("/") or all(
-            [obj.replace(_pref, "", 1).startswith("/") for obj in source_objects]
+            obj.replace(_pref, "", 1).startswith("/") for obj in source_objects
         )
 
         if is_directory:
@@ -451,7 +450,7 @@ class GCSToGCSOperator(BaseOperator):
             )
 
     def _check_exact_match(self, source_object: str, prefix: str) -> bool:
-        """Checks whether source_object's name matches the prefix according to the exact_match flag."""
+        """Check whether source_object's name matches the prefix according to the exact_match flag."""
         if self.exact_match and (source_object != prefix or not source_object.endswith(prefix)):
             return False
         return True
@@ -555,7 +554,8 @@ class GCSToGCSOperator(BaseOperator):
 
     def get_openlineage_facets_on_complete(self, task_instance):
         """
-        Implementing _on_complete because execute method does preprocessing on internals.
+        Implement _on_complete because execute method does preprocessing on internals.
+
         This means we won't have to normalize self.source_object and self.source_objects,
         destination bucket and so on.
         """
