@@ -15,14 +15,13 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-"""
-This module contains a mechanism for providing temporary
-Google Cloud authentication.
-"""
+"""This module contains a mechanism for providing temporary Google Cloud authentication."""
+
 from __future__ import annotations
 
 import json
 import logging
+import os
 import tempfile
 from contextlib import ExitStack, contextmanager
 from typing import Collection, Generator, Sequence
@@ -31,7 +30,7 @@ from urllib.parse import urlencode
 import google.auth
 import google.auth.credentials
 import google.oauth2.service_account
-from google.auth import impersonated_credentials
+from google.auth import impersonated_credentials  # type: ignore[attr-defined]
 from google.auth.environment_vars import CREDENTIALS, LEGACY_PROJECT, PROJECT
 
 from airflow.exceptions import AirflowException
@@ -51,8 +50,7 @@ def build_gcp_conn(
     project_id: str | None = None,
 ) -> str:
     """
-    Builds a uri that can be used as :envvar:`AIRFLOW_CONN_{CONN_ID}` with provided service key,
-    scopes and project id.
+    Build a uri that can be used as :envvar:`AIRFLOW_CONN_{CONN_ID}` with provided values.
 
     :param key_file_path: Path to service key.
     :param scopes: Required OAuth scopes.
@@ -79,8 +77,10 @@ def provide_gcp_credentials(
     key_file_dict: dict | None = None,
 ) -> Generator[None, None, None]:
     """
-    Context manager that provides a Google Cloud credentials for application supporting
-    `Application Default Credentials (ADC) strategy`__.
+    Context manager that provides Google Cloud credentials for Application Default Credentials (ADC).
+
+    .. seealso::
+        `Application Default Credentials (ADC) strategy`__.
 
     It can be used to provide credentials for external programs (e.g. gcloud) that expect authorization
     file in ``GOOGLE_APPLICATION_CREDENTIALS`` environment variable.
@@ -116,9 +116,9 @@ def provide_gcp_connection(
     project_id: str | None = None,
 ) -> Generator[None, None, None]:
     """
-    Context manager that provides a temporary value of :envvar:`AIRFLOW_CONN_GOOGLE_CLOUD_DEFAULT`
-    connection. It build a new connection that includes path to provided service json,
-    required scopes and project id.
+    Context manager that provides a temporary value of :envvar:`AIRFLOW_CONN_GOOGLE_CLOUD_DEFAULT` connection.
+
+    It builds a new connection that includes path to provided service json, required scopes and project id.
 
     :param key_file_path: Path to file with Google Cloud Service Account .json file.
     :param scopes: OAuth scopes for the connection
@@ -140,7 +140,9 @@ def provide_gcp_conn_and_credentials(
     project_id: str | None = None,
 ) -> Generator[None, None, None]:
     """
-    Context manager that provides both:
+    Context manager that provides GPC connection and credentials.
+
+     It provides both:
 
     - Google Cloud credentials for application supporting `Application Default Credentials (ADC)
       strategy`__.
@@ -166,7 +168,7 @@ def provide_gcp_conn_and_credentials(
 
 class _CredentialProvider(LoggingMixin):
     """
-    Prepare the Credentials object for Google API and the associated project_id
+    Prepare the Credentials object for Google API and the associated project_id.
 
     Only either `key_path` or `keyfile_dict` should be provided, or an exception will
     occur. If neither of them are provided, return default credentials for the current environment
@@ -196,6 +198,7 @@ class _CredentialProvider(LoggingMixin):
         self,
         key_path: str | None = None,
         keyfile_dict: dict[str, str] | None = None,
+        credential_config_file: dict[str, str] | str | None = None,
         key_secret_name: str | None = None,
         key_secret_project_id: str | None = None,
         scopes: Collection[str] | None = None,
@@ -213,6 +216,7 @@ class _CredentialProvider(LoggingMixin):
             )
         self.key_path = key_path
         self.keyfile_dict = keyfile_dict
+        self.credential_config_file = credential_config_file
         self.key_secret_name = key_secret_name
         self.key_secret_project_id = key_secret_project_id
         self.scopes = scopes
@@ -233,6 +237,8 @@ class _CredentialProvider(LoggingMixin):
             credentials, project_id = self._get_credentials_using_key_secret_name()
         elif self.keyfile_dict:
             credentials, project_id = self._get_credentials_using_keyfile_dict()
+        elif self.credential_config_file:
+            credentials, project_id = self._get_credentials_using_credential_config_file()
         else:
             credentials, project_id = self._get_credentials_using_adc()
 
@@ -311,9 +317,33 @@ class _CredentialProvider(LoggingMixin):
         project_id = credentials.project_id
         return credentials, project_id
 
+    def _get_credentials_using_credential_config_file(self):
+        if isinstance(self.credential_config_file, str) and os.path.exists(self.credential_config_file):
+            self._log_info(
+                f"Getting connection using credential configuration file: `{self.credential_config_file}`"
+            )
+            credentials, project_id = google.auth.load_credentials_from_file(
+                self.credential_config_file, scopes=self.scopes
+            )
+        else:
+            with tempfile.NamedTemporaryFile(mode="w+t") as temp_credentials_fd:
+                if isinstance(self.credential_config_file, dict):
+                    self._log_info("Getting connection using credential configuration dict.")
+                    temp_credentials_fd.write(json.dumps(self.credential_config_file))
+                elif isinstance(self.credential_config_file, str):
+                    self._log_info("Getting connection using credential configuration string.")
+                    temp_credentials_fd.write(self.credential_config_file)
+
+                temp_credentials_fd.flush()
+                credentials, project_id = google.auth.load_credentials_from_file(
+                    temp_credentials_fd.name, scopes=self.scopes
+                )
+
+        return credentials, project_id
+
     def _get_credentials_using_adc(self):
         self._log_info(
-            "Getting connection using `google.auth.default()` since no key file is defined for hook."
+            "Getting connection using `google.auth.default()` since no explicit credentials are provided."
         )
         credentials, project_id = google.auth.default(scopes=self.scopes)
         return credentials, project_id
@@ -328,14 +358,13 @@ class _CredentialProvider(LoggingMixin):
 
 
 def get_credentials_and_project_id(*args, **kwargs) -> tuple[google.auth.credentials.Credentials, str]:
-    """Returns the Credentials object for Google API and the associated project_id."""
+    """Return the Credentials object for Google API and the associated project_id."""
     return _CredentialProvider(*args, **kwargs).get_credentials_and_project()
 
 
 def _get_scopes(scopes: str | None = None) -> Sequence[str]:
     """
-    Parse a comma-separated string containing OAuth2 scopes if `scopes` is provided.
-    Otherwise, default scope will be returned.
+    Parse a comma-separated string containing OAuth2 scopes if `scopes` is provided; otherwise return default.
 
     :param scopes: A comma-separated string containing OAuth2 scopes
     :return: Returns the scope defined in the connection configuration, or the default scope
@@ -347,6 +376,8 @@ def _get_target_principal_and_delegates(
     impersonation_chain: str | Sequence[str] | None = None,
 ) -> tuple[str | None, Sequence[str] | None]:
     """
+    Get the target_principal and optional list of delegates from impersonation_chain.
+
     Analyze contents of impersonation_chain and return target_principal (the service account
     to directly impersonate using short-term credentials, if any) and optional list of delegates
     required to get the access_token of target_principal.
@@ -367,7 +398,7 @@ def _get_target_principal_and_delegates(
 
 def _get_project_id_from_service_account_email(service_account_email: str) -> str:
     """
-    Extracts project_id from service account's email address.
+    Extract project_id from service account's email address.
 
     :param service_account_email: email of the service account.
 
