@@ -20,12 +20,15 @@ from __future__ import annotations
 import posixpath
 from functools import wraps
 from shutil import copyfileobj
+from typing import TYPE_CHECKING, Any
 
 import smbclient
-import smbprotocol.connection
 
 from airflow.hooks.base import BaseHook
 
+if TYPE_CHECKING:
+    import smbprotocol.connection
+
 
 class SambaHook(BaseHook):
     """Allows for interaction with a Samba server.
@@ -246,3 +249,11 @@ class SambaHook(BaseHook):
         """Push local file to samba server."""
         with open(local_filepath, "rb") as f, self.open_file(destination_filepath, mode="wb") as g:
             copyfileobj(f, g)
+
+    @classmethod
+    def get_ui_field_behaviour(cls) -> dict[str, Any]:
+        """Return custom field behaviour."""
+        return {
+            "hidden_fields": ["extra"],
+            "relabeling": {"schema": "Share"},
+        }
