@@ -20,6 +20,7 @@ Builds documentation and runs spell checking
 
 # isort:skip_file (needed to workaround isort bug)
 """
+
 from __future__ import annotations
 
 import argparse
@@ -28,7 +29,7 @@ import multiprocessing
 import os
 import sys
 from collections import defaultdict
-from typing import Callable, Iterable, NamedTuple, TypeVar
+from typing import Any, Callable, Iterable, NamedTuple, TypeVar
 
 from rich.console import Console
 from tabulate import tabulate
@@ -185,7 +186,8 @@ class SpellCheckResult(NamedTuple):
 
     package_name: str
     log_file_name: str
-    errors: list[SpellingError]
+    spelling_errors: list[SpellingError]
+    build_errors: list[DocBuildError]
 
 
 def perform_docs_build_for_single_package(build_specification: BuildSpecification) -> BuildDocsResult:
@@ -206,11 +208,13 @@ def perform_spell_check_for_single_package(build_specification: BuildSpecificati
     """Performs single package spell check."""
     builder = AirflowDocsBuilder(package_name=build_specification.package_name)
     console.print(f"[info]{build_specification.package_name:60}:[/] Checking spelling started")
+    spelling_errors, build_errors = builder.check_spelling(
+        verbose=build_specification.verbose,
+    )
     result = SpellCheckResult(
         package_name=build_specification.package_name,
-        errors=builder.check_spelling(
-            verbose=build_specification.verbose,
-        ),
+        spelling_errors=spelling_errors,
+        build_errors=build_errors,
         log_file_name=builder.log_spelling_filename,
     )
     console.print(f"[info]{build_specification.package_name:60}:[/] Checking spelling completed")
@@ -218,7 +222,7 @@ def perform_spell_check_for_single_package(build_specification: BuildSpecificati
 
 
 def build_docs_for_packages(
-    current_packages: list[str],
+    packages_to_build: list[str],
     docs_only: bool,
     spellcheck_only: bool,
     jobs: int,
@@ -228,28 +232,28 @@ def build_docs_for_packages(
     all_build_errors: dict[str, list[DocBuildError]] = defaultdict(list)
     all_spelling_errors: dict[str, list[SpellingError]] = defaultdict(list)
     with with_group("Cleaning documentation files"):
-        for package_name in current_packages:
+        for package_name in packages_to_build:
             console.print(f"[info]{package_name:60}:[/] Cleaning files")
             builder = AirflowDocsBuilder(package_name=package_name)
             builder.clean_files()
     if jobs > 1:
         run_in_parallel(
-            all_build_errors,
-            all_spelling_errors,
-            current_packages,
-            docs_only,
-            jobs,
-            spellcheck_only,
-            verbose,
+            all_build_errors=all_build_errors,
+            all_spelling_errors=all_spelling_errors,
+            packages_to_build=packages_to_build,
+            docs_only=docs_only,
+            jobs=jobs,
+            spellcheck_only=spellcheck_only,
+            verbose=verbose,
         )
     else:
         run_sequentially(
-            all_build_errors,
-            all_spelling_errors,
-            current_packages,
-            docs_only,
-            spellcheck_only,
-            verbose,
+            all_build_errors=all_build_errors,
+            all_spelling_errors=all_spelling_errors,
+            packages_to_build=packages_to_build,
+            docs_only=docs_only,
+            spellcheck_only=spellcheck_only,
+            verbose=verbose,
         )
     return all_build_errors, all_spelling_errors
 
@@ -257,14 +261,14 @@ def build_docs_for_packages(
 def run_sequentially(
     all_build_errors,
     all_spelling_errors,
-    current_packages,
+    packages_to_build,
     docs_only,
     spellcheck_only,
     verbose,
 ):
     """Run both - spellcheck and docs build sequentially without multiprocessing"""
     if not spellcheck_only:
-        for package_name in current_packages:
+        for package_name in packages_to_build:
             build_result = perform_docs_build_for_single_package(
                 build_specification=BuildSpecification(
                     package_name=package_name,
@@ -275,40 +279,43 @@ def run_sequentially(
                 all_build_errors[package_name].extend(build_result.errors)
                 print_build_output(build_result)
     if not docs_only:
-        for package_name in current_packages:
+        for package_name in packages_to_build:
             spellcheck_result = perform_spell_check_for_single_package(
                 build_specification=BuildSpecification(
                     package_name=package_name,
                     verbose=verbose,
                 )
             )
-            if spellcheck_result.errors:
-                all_spelling_errors[package_name].extend(spellcheck_result.errors)
+            if spellcheck_result.spelling_errors:
+                all_spelling_errors[package_name].extend(spellcheck_result.spelling_errors)
+                if spellcheck_only:
+                    all_build_errors[package_name].extend(spellcheck_result.build_errors)
                 print_spelling_output(spellcheck_result)
 
 
 def run_in_parallel(
-    all_build_errors,
-    all_spelling_errors,
-    current_packages,
-    docs_only,
-    jobs,
-    spellcheck_only,
-    verbose,
+    all_build_errors: dict[str, list[DocBuildError]],
+    all_spelling_errors: dict[str, list[SpellingError]],
+    packages_to_build: list[str],
+    docs_only: bool,
+    jobs: int,
+    spellcheck_only: bool,
+    verbose: bool,
 ):
     """Run both - spellcheck and docs build sequentially without multiprocessing"""
     with multiprocessing.Pool(processes=jobs) as pool:
         if not spellcheck_only:
             run_docs_build_in_parallel(
                 all_build_errors=all_build_errors,
-                current_packages=current_packages,
+                packages_to_build=packages_to_build,
                 verbose=verbose,
                 pool=pool,
             )
         if not docs_only:
             run_spell_check_in_parallel(
                 all_spelling_errors=all_spelling_errors,
-                current_packages=current_packages,
+                all_build_errors=all_build_errors,
+                packages_to_build=packages_to_build,
                 verbose=verbose,
                 pool=pool,
             )
@@ -327,14 +334,14 @@ def print_build_output(result: BuildDocsResult):
 
 def run_docs_build_in_parallel(
     all_build_errors: dict[str, list[DocBuildError]],
-    current_packages: list[str],
+    packages_to_build: list[str],
     verbose: bool,
-    pool,
+    pool: Any,  # Cannot use multiprocessing types here: https://github.com/python/typeshed/issues/4266
 ):
     """Runs documentation building in parallel."""
     doc_build_specifications: list[BuildSpecification] = []
     with with_group("Scheduling documentation to build"):
-        for package_name in current_packages:
+        for package_name in packages_to_build:
             console.print(f"[info]{package_name:60}:[/] Scheduling documentation to build")
             doc_build_specifications.append(
                 BuildSpecification(
@@ -365,22 +372,24 @@ def print_spelling_output(result: SpellCheckResult):
 
 def run_spell_check_in_parallel(
     all_spelling_errors: dict[str, list[SpellingError]],
-    current_packages: list[str],
+    all_build_errors: dict[str, list[DocBuildError]],
+    packages_to_build: list[str],
     verbose: bool,
     pool,
 ):
     """Runs spell check in parallel."""
     spell_check_specifications: list[BuildSpecification] = []
     with with_group("Scheduling spell checking of documentation"):
-        for package_name in current_packages:
+        for package_name in packages_to_build:
             console.print(f"[info]{package_name:60}:[/] Scheduling spellchecking")
             spell_check_specifications.append(BuildSpecification(package_name=package_name, verbose=verbose))
     with with_group("Running spell checking of documentation"):
         console.print()
         result_list = pool.map(perform_spell_check_for_single_package, spell_check_specifications)
     for result in result_list:
-        if result.errors:
-            all_spelling_errors[result.package_name].extend(result.errors)
+        if result.spelling_errors:
+            all_spelling_errors[result.package_name].extend(result.spelling_errors)
+            all_build_errors[result.package_name].extend(result.build_errors)
             print_spelling_output(result)
 
 
@@ -405,12 +414,17 @@ def display_packages_summary(
 def print_build_errors_and_exit(
     build_errors: dict[str, list[DocBuildError]],
     spelling_errors: dict[str, list[SpellingError]],
+    spellcheck_only: bool,
 ) -> None:
     """Prints build errors and exists."""
     if build_errors or spelling_errors:
         if build_errors:
-            display_errors_summary(build_errors)
-            console.print()
+            if spellcheck_only:
+                console.print("f[warning]There were some build errors remaining.")
+                console.print()
+            else:
+                display_errors_summary(build_errors)
+                console.print()
         if spelling_errors:
             display_spelling_error_summary(spelling_errors)
             console.print()
@@ -438,21 +452,21 @@ def main():
 
     if package_filters:
         console.print("Current package filters: ", package_filters)
-    current_packages = process_package_filters(available_packages, package_filters)
+    packages_to_build = process_package_filters(available_packages, package_filters)
     with with_group("Fetching inventories"):
         # Inventories that could not be retrieved should be built first. This may mean this is a
         # new package.
         packages_without_inventories = fetch_inventories()
     normal_packages, priority_packages = partition(
-        lambda d: d in packages_without_inventories, current_packages
+        lambda d: d in packages_without_inventories, packages_to_build
     )
     normal_packages, priority_packages = list(normal_packages), list(priority_packages)
     jobs = args.jobs if args.jobs != 0 else os.cpu_count()
 
     with with_group(
-        f"Documentation will be built for {len(current_packages)} package(s) with {jobs} parallel jobs"
+        f"Documentation will be built for {len(packages_to_build)} package(s) with {jobs} parallel jobs"
     ):
-        for pkg_no, pkg in enumerate(current_packages, start=1):
+        for pkg_no, pkg in enumerate(packages_to_build, start=1):
             console.print(f"{pkg_no}. {pkg}")
 
     all_build_errors: dict[str | None, list[DocBuildError]] = {}
@@ -460,7 +474,7 @@ def main():
     if priority_packages:
         # Build priority packages
         package_build_errors, package_spelling_errors = build_docs_for_packages(
-            current_packages=priority_packages,
+            packages_to_build=priority_packages,
             docs_only=docs_only,
             spellcheck_only=spellcheck_only,
             jobs=jobs,
@@ -476,7 +490,7 @@ def main():
     # two or more inventories, it is better to try to build for all packages as the previous packages
     # may have failed as well.
     package_build_errors, package_spelling_errors = build_docs_for_packages(
-        current_packages=current_packages if len(priority_packages) > 1 else normal_packages,
+        packages_to_build=packages_to_build if len(priority_packages) > 1 else normal_packages,
         docs_only=docs_only,
         spellcheck_only=spellcheck_only,
         jobs=jobs,
@@ -489,29 +503,50 @@ def main():
 
     if not args.one_pass_only:
         # Build documentation for some packages again if it can help them.
-        package_build_errors, package_spelling_errors = retry_building_docs_if_needed(
-            all_build_errors,
-            all_spelling_errors,
-            args,
-            docs_only,
-            jobs,
-            package_build_errors,
-            package_spelling_errors,
-            spellcheck_only,
+        package_build_errors = retry_building_docs_if_needed(
+            all_build_errors=all_build_errors,
+            all_spelling_errors=all_spelling_errors,
+            args=args,
+            docs_only=docs_only,
+            jobs=jobs,
+            package_build_errors=package_build_errors,
+            originally_built_packages=packages_to_build,
+            # If spellchecking fails, we need to rebuild all packages first in case some references
+            # are broken between packages
+            rebuild_all_packages=spellcheck_only,
         )
 
-        # And try again in case one change spans across three-level dependencies
-        retry_building_docs_if_needed(
-            all_build_errors,
-            all_spelling_errors,
-            args,
-            docs_only,
-            jobs,
-            package_build_errors,
-            package_spelling_errors,
-            spellcheck_only,
+        # And try again in case one change spans across three-level dependencies.
+        package_build_errors = retry_building_docs_if_needed(
+            all_build_errors=all_build_errors,
+            all_spelling_errors=all_spelling_errors,
+            args=args,
+            docs_only=docs_only,
+            jobs=jobs,
+            package_build_errors=package_build_errors,
+            originally_built_packages=packages_to_build,
+            # In the 3rd pass we only rebuild packages that failed in the 2nd pass
+            # no matter if we do spellcheck-only build
+            rebuild_all_packages=False,
         )
 
+        if spellcheck_only:
+            # And in case of spellcheck-only, we add a 4th pass to account for A->B-C case
+            # For spellcheck-only build, the first pass does not solve any of the dependency
+            # Issues, they only start getting solved and the 2nd pass so we might need to do one more pass
+            package_build_errors = retry_building_docs_if_needed(
+                all_build_errors=all_build_errors,
+                all_spelling_errors=all_spelling_errors,
+                args=args,
+                docs_only=docs_only,
+                jobs=jobs,
+                package_build_errors=package_build_errors,
+                originally_built_packages=packages_to_build,
+                # In the 4th pass we only rebuild packages that failed in the 3rd pass
+                # no matter if we do spellcheck-only build
+                rebuild_all_packages=False,
+            )
+
     if not disable_checks:
         general_errors = lint_checks.run_all_check(disable_provider_checks=disable_provider_checks)
         if general_errors:
@@ -528,44 +563,52 @@ def main():
     print_build_errors_and_exit(
         all_build_errors,
         all_spelling_errors,
+        spellcheck_only,
     )
 
 
 def retry_building_docs_if_needed(
-    all_build_errors,
-    all_spelling_errors,
-    args,
-    docs_only,
-    jobs,
-    package_build_errors,
-    package_spelling_errors,
-    spellcheck_only,
-):
+    all_build_errors: dict[str, list[DocBuildError]],
+    all_spelling_errors: dict[str, list[SpellingError]],
+    args: argparse.Namespace,
+    docs_only: bool,
+    jobs: int,
+    package_build_errors: dict[str, list[DocBuildError]],
+    originally_built_packages: list[str],
+    rebuild_all_packages: bool,
+) -> dict[str, list[DocBuildError]]:
     to_retry_packages = [
         package_name
         for package_name, errors in package_build_errors.items()
         if any(any((m in e.message) for m in ERRORS_ELIGIBLE_TO_REBUILD) for e in errors)
     ]
-    if to_retry_packages:
-        for package_name in to_retry_packages:
-            if package_name in all_build_errors:
-                del all_build_errors[package_name]
-            if package_name in all_spelling_errors:
-                del all_spelling_errors[package_name]
-
-        package_build_errors, package_spelling_errors = build_docs_for_packages(
-            current_packages=to_retry_packages,
-            docs_only=docs_only,
-            spellcheck_only=spellcheck_only,
-            jobs=jobs,
-            verbose=args.verbose,
-        )
-        if package_build_errors:
-            all_build_errors.update(package_build_errors)
-        if package_spelling_errors:
-            all_spelling_errors.update(package_spelling_errors)
-        return package_build_errors, package_spelling_errors
-    return package_build_errors, package_spelling_errors
+    if not to_retry_packages:
+        console.print("[green]No packages to retry. No more passes are needed.[/]")
+        return package_build_errors
+    console.print("[warning] Some packages failed to build due to dependencies. We need another pass.[/]")
+    # if we are rebuilding all packages, we need to retry all packages
+    # even if there is one package to rebuild only
+    if rebuild_all_packages:
+        console.print("[warning]Rebuilding all originally built package as this is the first build pass:[/]")
+        to_retry_packages = originally_built_packages
+    console.print(f"[bright_blue]Packages to rebuild: {to_retry_packages}[/]")
+    for package_name in to_retry_packages:
+        if package_name in all_build_errors:
+            del all_build_errors[package_name]
+        if package_name in all_spelling_errors:
+            del all_spelling_errors[package_name]
+    package_build_errors, package_spelling_errors = build_docs_for_packages(
+        packages_to_build=to_retry_packages,
+        docs_only=docs_only,
+        spellcheck_only=False,
+        jobs=jobs,
+        verbose=args.verbose,
+    )
+    if package_build_errors:
+        all_build_errors.update(package_build_errors)
+    if package_spelling_errors:
+        all_spelling_errors.update(package_spelling_errors)
+    return package_build_errors
 
 
 if __name__ == "__main__":
