@@ -70,10 +70,12 @@ class TestPodTemplateFile:
                     "gitSync": {
                         "enabled": True,
                         "containerName": "git-sync-test",
-                        "wait": 66,
+                        "wait": None,
+                        "period": "66s",
                         "maxFailures": 70,
                         "subPath": "path1/path2",
                         "rev": "HEAD",
+                        "ref": "test-branch",
                         "depth": 1,
                         "repo": "https://github.com/apache/airflow.git",
                         "branch": "test-branch",
@@ -95,15 +97,23 @@ class TestPodTemplateFile:
             "imagePullPolicy": "Always",
             "env": [
                 {"name": "GIT_SYNC_REV", "value": "HEAD"},
+                {"name": "GITSYNC_REF", "value": "test-branch"},
                 {"name": "GIT_SYNC_BRANCH", "value": "test-branch"},
                 {"name": "GIT_SYNC_REPO", "value": "https://github.com/apache/airflow.git"},
+                {"name": "GITSYNC_REPO", "value": "https://github.com/apache/airflow.git"},
                 {"name": "GIT_SYNC_DEPTH", "value": "1"},
+                {"name": "GITSYNC_DEPTH", "value": "1"},
                 {"name": "GIT_SYNC_ROOT", "value": "/git"},
+                {"name": "GITSYNC_ROOT", "value": "/git"},
                 {"name": "GIT_SYNC_DEST", "value": "repo"},
+                {"name": "GITSYNC_LINK", "value": "repo"},
                 {"name": "GIT_SYNC_ADD_USER", "value": "true"},
-                {"name": "GIT_SYNC_WAIT", "value": "66"},
+                {"name": "GITSYNC_ADD_USER", "value": "true"},
+                {"name": "GITSYNC_PERIOD", "value": "66s"},
                 {"name": "GIT_SYNC_MAX_SYNC_FAILURES", "value": "70"},
+                {"name": "GITSYNC_MAX_FAILURES", "value": "70"},
                 {"name": "GIT_SYNC_ONE_TIME", "value": "true"},
+                {"name": "GITSYNC_ONE_TIME", "value": "true"},
             ],
             "volumeMounts": [{"mountPath": "/git", "name": "dags"}],
             "resources": {},
@@ -185,12 +195,21 @@ class TestPodTemplateFile:
         assert {"name": "GIT_SSH_KEY_FILE", "value": "/etc/git-secret/ssh"} in jmespath.search(
             "spec.initContainers[0].env", docs[0]
         )
+        assert {"name": "GITSYNC_SSH_KEY_FILE", "value": "/etc/git-secret/ssh"} in jmespath.search(
+            "spec.initContainers[0].env", docs[0]
+        )
         assert {"name": "GIT_SYNC_SSH", "value": "true"} in jmespath.search(
             "spec.initContainers[0].env", docs[0]
         )
+        assert {"name": "GITSYNC_SSH", "value": "true"} in jmespath.search(
+            "spec.initContainers[0].env", docs[0]
+        )
         assert {"name": "GIT_KNOWN_HOSTS", "value": "false"} in jmespath.search(
             "spec.initContainers[0].env", docs[0]
         )
+        assert {"name": "GITSYNC_SSH_KNOWN_HOSTS", "value": "false"} in jmespath.search(
+            "spec.initContainers[0].env", docs[0]
+        )
         assert {
             "name": "git-sync-ssh-key",
             "mountPath": "/etc/git-secret/ssh",
@@ -256,6 +275,16 @@ class TestPodTemplateFile:
             "valueFrom": {"secretKeyRef": {"name": "user-pass-secret", "key": "GIT_SYNC_PASSWORD"}},
         } in jmespath.search("spec.initContainers[0].env", docs[0])
 
+        # Testing git-sync v4
+        assert {
+            "name": "GITSYNC_USERNAME",
+            "valueFrom": {"secretKeyRef": {"name": "user-pass-secret", "key": "GITSYNC_USERNAME"}},
+        } in jmespath.search("spec.initContainers[0].env", docs[0])
+        assert {
+            "name": "GITSYNC_PASSWORD",
+            "valueFrom": {"secretKeyRef": {"name": "user-pass-secret", "key": "GITSYNC_PASSWORD"}},
+        } in jmespath.search("spec.initContainers[0].env", docs[0])
+
     def test_should_set_the_dags_volume_claim_correctly_when_using_an_existing_claim(self):
         docs = render_chart(
             values={"dags": {"persistence": {"enabled": True, "existingClaim": "test-claim"}}},
