@@ -25,7 +25,20 @@ from tests.charts.helm_template_generator import render_chart
 class TestWebserverDeployment:
     """Tests webserver deployment."""
 
-    def test_should_add_host_header_to_liveness_and_readiness_probes(self):
+    def test_can_be_disabled(self):
+        """
+        Webserver should be able to be disabled if the users desires.
+
+        For example, user may be disabled when using webserver and having it deployed on another host.
+        """
+        docs = render_chart(
+            values={"webserver": {"enabled": False}},
+            show_only=["templates/webserver/webserver-deployment.yaml"],
+        )
+
+        assert 0 == len(docs)
+
+    def test_should_add_host_header_to_liveness_and_readiness_and_startup_probes(self):
         docs = render_chart(
             values={
                 "config": {
@@ -41,8 +54,11 @@ class TestWebserverDeployment:
         assert {"name": "Host", "value": "example.com"} in jmespath.search(
             "spec.template.spec.containers[0].readinessProbe.httpGet.httpHeaders", docs[0]
         )
+        assert {"name": "Host", "value": "example.com"} in jmespath.search(
+            "spec.template.spec.containers[0].startupProbe.httpGet.httpHeaders", docs[0]
+        )
 
-    def test_should_add_path_to_liveness_and_readiness_probes(self):
+    def test_should_add_path_to_liveness_and_readiness_and_startup_probes(self):
         docs = render_chart(
             values={
                 "config": {
@@ -60,6 +76,10 @@ class TestWebserverDeployment:
             jmespath.search("spec.template.spec.containers[0].readinessProbe.httpGet.path", docs[0])
             == "/mypath/path/health"
         )
+        assert (
+            jmespath.search("spec.template.spec.containers[0].startupProbe.httpGet.path", docs[0])
+            == "/mypath/path/health"
+        )
 
     @pytest.mark.parametrize(
         "revision_history_limit, global_revision_history_limit",
@@ -91,6 +111,10 @@ class TestWebserverDeployment:
             jmespath.search("spec.template.spec.containers[0].readinessProbe.httpGet.httpHeaders", docs[0])
             is None
         )
+        assert (
+            jmespath.search("spec.template.spec.containers[0].startupProbe.httpGet.httpHeaders", docs[0])
+            is None
+        )
 
     def test_should_use_templated_base_url_for_probes(self):
         docs = render_chart(
@@ -111,15 +135,20 @@ class TestWebserverDeployment:
         assert {"name": "Host", "value": "release-name.com"} in jmespath.search(
             "readinessProbe.httpGet.httpHeaders", container
         )
+        assert {"name": "Host", "value": "release-name.com"} in jmespath.search(
+            "startupProbe.httpGet.httpHeaders", container
+        )
         assert "/mypath/release-name/path/health" == jmespath.search("livenessProbe.httpGet.path", container)
         assert "/mypath/release-name/path/health" == jmespath.search("readinessProbe.httpGet.path", container)
+        assert "/mypath/release-name/path/health" == jmespath.search("startupProbe.httpGet.path", container)
 
-    def test_should_add_scheme_to_liveness_and_readiness_probes(self):
+    def test_should_add_scheme_to_liveness_and_readiness_and_startup_probes(self):
         docs = render_chart(
             values={
                 "webserver": {
                     "livenessProbe": {"scheme": "HTTPS"},
                     "readinessProbe": {"scheme": "HTTPS"},
+                    "startupProbe": {"scheme": "HTTPS"},
                 }
             },
             show_only=["templates/webserver/webserver-deployment.yaml"],
@@ -131,6 +160,9 @@ class TestWebserverDeployment:
         assert "HTTPS" in jmespath.search(
             "spec.template.spec.containers[0].readinessProbe.httpGet.scheme", docs[0]
         )
+        assert "HTTPS" in jmespath.search(
+            "spec.template.spec.containers[0].startupProbe.httpGet.scheme", docs[0]
+        )
 
     def test_should_add_volume_and_volume_mount_when_exist_webserver_config(self):
         docs = render_chart(
@@ -423,6 +455,17 @@ class TestWebserverDeployment:
             "spec.template.spec.topologySpreadConstraints[0]", docs[0]
         )
 
+    def test_scheduler_name(self):
+        docs = render_chart(
+            values={"schedulerName": "airflow-scheduler"},
+            show_only=["templates/webserver/webserver-deployment.yaml"],
+        )
+
+        assert "airflow-scheduler" == jmespath.search(
+            "spec.template.spec.schedulerName",
+            docs[0],
+        )
+
     @pytest.mark.parametrize(
         "log_persistence_values, expected_claim_name",
         [
@@ -760,6 +803,20 @@ class TestWebserverDeployment:
         assert "127.0.0.1" == jmespath.search("spec.template.spec.hostAliases[0].ip", docs[0])
         assert "foo.local" == jmespath.search("spec.template.spec.hostAliases[0].hostnames[0]", docs[0])
 
+    def test_should_add_annotations_to_webserver_configmap(self):
+        docs = render_chart(
+            values={
+                "webserver": {
+                    "webserverConfig": "CSRF_ENABLED = True  # {{ .Release.Name }}",
+                    "configMapAnnotations": {"test_annotation": "test_annotation_value"},
+                },
+            },
+            show_only=["templates/configmaps/webserver-configmap.yaml"],
+        )
+
+        assert "annotations" in jmespath.search("metadata", docs[0])
+        assert jmespath.search("metadata.annotations", docs[0])["test_annotation"] == "test_annotation_value"
+
 
 class TestWebserverService:
     """Tests webserver service."""
